library IEEE;
use IEEE.STD_LOGIC_1164.all;
--use IEEE.STD_LOGIC_ARITH.all;
--use IEEE.STD_LOGIC_SIGNED.all;
use IEEE.numeric_std.all;

-------------------------------------------------------------------------------
-- entity
-------------------------------------------------------------------------------

entity de2_audio is  
port (
    clk : in std_logic;
    reset_n : in std_logic;

    -- Bus master signals
    address      : in std_logic_vector (7 downto 0);
    byteenable   : in std_logic_vector (1 downto 0);
    writedata    : in std_logic_vector (15 downto 0);
    read         : in std_logic;
    write        : in std_logic;
    chipselect   : in std_logic;

     -- Slave signals
    readdata     : out std_logic_vector (15 downto 0);
    waitrequest  : out std_logic;    
    
    -- Audio interface signals
    AUD_CLK	     : in std_logic;                     --    18.43MHz audio clock AUD_XCK
    AUD_ADCLRCK  : out std_logic;                    --    Audio CODEC ADC LR Clock
    AUD_ADCDAT   : in  std_logic;                    --    Audio CODEC ADC Data
    AUD_DACLRCK  : out std_logic;                    --    Audio CODEC DAC LR Clock
    AUD_DACDAT   : out std_logic;                    --    Audio CODEC DAC Data
    AUD_BCLK     : inout std_logic;                  --    Audio CODEC Bit-Stream Clock
    AUD_XCK		 : out std_logic;
    -- Test signals --
    ledr	: out std_logic_vector (17 downto 0)
  );
end entity de2_audio; 

-------------------------------------------------------------------------------
-- architecture
-------------------------------------------------------------------------------

architecture imp of de2_audio is 

	  component de2_fifo
	  port (
			data		: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
			rdclk		: IN STD_LOGIC ;
			rdreq		: IN STD_LOGIC ;
			wrclk		: IN STD_LOGIC ;
			wrreq		: IN STD_LOGIC ;
			q			: OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
			rdempty		: OUT STD_LOGIC ;
			wrfull		: OUT STD_LOGIC ;
			wrusedw		: OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
	  );
	  end component;

	component de2_wm8731_audio_in is
	port (
		clk : in std_logic;       --  Audio CODEC Chip Clock AUD_XCK (18.43 MHz)
		reset_n : in std_logic;
		data_out : out std_logic_vector(15 downto 0);
		audio_req : out std_logic;

		-- Audio interface signals
		AUD_ADCLRCK  : out std_logic;   --    Audio CODEC ADC LR Clock
		AUD_ADCDAT   : in  std_logic;   --    Audio CODEC ADC Data
		AUD_BCLK     : inout std_logic  --    Audio CODEC Bit-Stream Clock
	);
	end component;
	
	component de2_wm8731_audio_out is
	port (
		clk : in std_logic;       --  Audio CODEC Chip Clock AUD_XCK (18.43 MHz)
		reset_n : in std_logic;
		test_mode : in std_logic;       --    Audio CODEC controller test mode
		data_in : in std_logic_vector(15 downto 0);	
		audio_req : out std_logic;
	  
		-- Audio interface signals
		AUD_DACLRCK  : out std_logic;   --    Audio CODEC DAC LR Clock
		AUD_DACDAT   : out std_logic   --    Audio CODEC DAC Data
	);
	end component;	

	signal reset : std_logic;

	signal dac_request : std_logic;
	signal adc_request : std_logic;
	signal write_fifo : std_logic;
	signal data_from_bus : std_logic_vector (15 downto 0);
	signal data_to_bus : std_logic_vector (15 downto 0);
	signal adcdata : std_logic_vector (15 downto 0);
	signal dacdata : std_logic_vector (15 downto 0);
	signal rdempty : std_logic;
	signal wrfull : std_logic;
	signal writefifo   : std_logic;
	signal readfifo	   : std_logic;	

	signal counter : std_logic_vector (13 downto 0);
	signal sin_counter : std_logic_vector (5 downto 0);
	signal sin_out : std_logic_vector (15 downto 0);

	signal rate : std_logic_vector(2 downto 0); -- "XX1" 11kHz, "01X" 22kHz, "1XX" 44kHz
	signal rate_counter : std_logic_vector (2 downto 0);
	signal req_fifo : std_logic;

	signal used_buf : std_logic_vector (7 downto 0);

	--test
	signal ledr_count : std_logic_vector (16 downto 0);
	signal led_ctrl : std_logic;
	
	--Audio clk dividers
	--signal audio_clock : unsigned(1 downto 0) := "00";
	signal audio_request : std_logic;
	signal counter_aud_adclkrck : unsigned (15 downto 0);
	signal LRCK: std_logic;
	
begin

--  assignments
--  ledr(16 downto 0) <= ledr_count(16 downto 0);
--  ledr(16 downto 13) <= address(3 downto 0);
--  ledr(17) <= wrempty;

--  process (clk, reset_n)
--  begin
--    if (clk'event and clk = '1') then
--      writefifo <= '0';
--      if(write = '1' and chipselect = '1') then
--        if(address = "0000") then
--          data_from_bus <= writedata;
--          writefifo <= '1';
--        else
--          rate <= writedata(13 downto 11);
--        end if;
--      end if;
--      if(read = '1' and chipselect = '1' and address = "0000") then
--        readdata <= "000"& used_buf;
--      end if;
--    end if;
--  end process;

--  process (klcoidua)
--  begin
--    if(klcoidua'event and klcoidua = '1') then
--      req_fifo <= '0';
--      if(sound_request = '1') then
--        if((rate_counter(2) = '1' and rate(2) = '1') or
--           (rate_counter(1) = '1' and rate(1) = '1') or
--           (rate_counter(0) = '1' and rate(0) = '1')) then
--          rate_counter <= "100";
--          req_fifo <= '1';
--        else
--          rate_counter <= '0' & rate_counter (2 downto 1);
--        end if;
--      end if;
--    end if;
--  end process;
--
--  process(wrempty)
--	begin
--	  if(wrempty'event and wrempty = '1') then
--        ledr_count <= ledr_count + '1';
--      end if;
--  end process;

	BusComm : process (clk)
	begin
	  if rising_edge (clk) then
		writefifo <= '0';	
		if reset = '1' then
			readdata <= (others => '0');
		else
			if chipselect = '1' then
			  if address (0) = '0' then
				  if write = '1' then
					data_from_bus <= writedata;	
					writefifo <= '1';				
				  elsif read = '1' then
					readdata <= data_to_bus;
				  end if;
--			  elsif address(0) = '1' then
--				  if write = '1' then
--					RECTANGLE_VSTART <= to_integer(writedata);
--				  elsif read = '1' then
--				    readdata <= to_unsigned(RECTANGLE_VSTART, 16);
--				  end if;
			  end if;
			end if;
		end if;
	  end if;
	end process BusComm;
		
	process(AUD_BCLK)
	begin
	if rising_edge(AUD_BCLK) then
		counter_aud_adclkrck <= counter_aud_adclkrck + 1;
	end if;

	if (counter_aud_adclkrck = "0000000000000000") then
		led_ctrl <= not led_ctrl;
	end if;
	end process;
	
--	-- 12.5 MHz clk
--	process (clk)
--	begin
--		if rising_edge(clk) then
--			audio_clock <= audio_clock + "1";
--		end if;
--	end process;   

	AUD_XCK <= AUD_CLK;

    ADC : de2_wm8731_audio_in
    port map (
		clk => AUD_CLK,
		reset_n => reset_n,
		data_out => data_to_bus, --adcdata
		audio_req => adc_request,

		AUD_ADCLRCK => AUD_ADCLRCK,
		AUD_ADCDAT => AUD_ADCDAT,
		AUD_BCLK => AUD_BCLK
	);
	
	DAC : de2_wm8731_audio_out
    port map (
		clk => AUD_CLK,
		reset_n => reset_n,      
		test_mode => '0',
		audio_req => dac_request,
		data_in => dacdata,          

		AUD_DACLRCK  => AUD_DACLRCK,
		AUD_DACDAT   => AUD_DACDAT
	);  

--    the_fifo : fifo
--    the_fifo : aud_fifo
--    port map (
--		data		=> data_from_bus,
--		rdclk		=> klcoidua,
--		rdreq		=> req_fifo,
--		wrclk		=> clk,
--		wrreq		=> writefifo,
--		q			=> mdata,
--		rdempty		=> rdempty,
--		wrempty		=> wrempty,
--		wrusedw		=> used_buf		
--	);

--	ADC_FIFO : de2_fifo
--	port map (
--		data		=> adcdata,
--		rdclk		=> clk,
--		rdreq		=> readfifo, -- define this
--		wrclk		=> AUD_CLK,
--		wrreq		=> adc_request,
--		q		    => data_to_bus,
--		rdempty		=> rdempty,
--		wrfull		=> wrfull,
--		wrusedw		=> used_buf	
--	);


	DAC_FIFO : de2_fifo
	port map (
		data		=> data_from_bus,
		rdclk		=> AUD_CLK,
		rdreq		=> dac_request,
		wrclk		=> clk,
		wrreq		=> writefifo,
		q		    => dacdata,
		rdempty		=> rdempty,
		wrfull		=> wrfull,
		wrusedw		=> used_buf	
	);		 


	  ledr(1)  <= led_ctrl;
	  ledr(2)  <= '0';
	  ledr(3)  <= '0';
	  ledr(0)  <= '0';

end architecture imp;
