#include "types.h"

#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include "fmtheaders.h"

#define inw(port) (*(volatile unsigned *)(port))
#define outw(d,port) (*(volatile unsigned *)(port))=(d)

#define IORD(address,offset) (*(volatile unsigned *)(((address)|0x80000000)+2*(offset)))
#define IOWR(address,offset,value) (*(volatile unsigned *)(((address)|0x80000000)+2*(offset)))=(value)

#define numSamples 262144
#define DELAY2 0

int timelimit = 40;
int dsp_speed = 4000; 
int dsp_stereo = 1;
int samplesize = 16;

/* write a WAVE-header */
void start_wave(int fd, unsigned long cnt)
{
  WaveHeader wh;

  wh.main_chunk = RIFF;
  wh.length     = cnt + sizeof(WaveHeader) - 8; 
  wh.chunk_type = WAVE;
  wh.sub_chunk  = FMT;
  wh.sc_len     = 16;
  wh.format     = PCM_CODE;
  wh.modus      = dsp_stereo ? 2 : 1;
  printf("stereo 2 or mono 1:%d\n", wh.modus);
  wh.sample_fq  = dsp_speed;
  wh.byte_p_spl = ((samplesize == 8) ? 1 : 2) * (dsp_stereo ? 2 : 1);
  wh.byte_p_sec = dsp_speed * wh.modus * wh.byte_p_spl;
  wh.bit_p_spl  = samplesize;
  wh.data_chunk = DATA;
  wh.data_length= cnt;
  fwrite (&wh, sizeof(WaveHeader), 1, fd);
}

unsigned long calc_count(void)
{
  unsigned long count;

  if (!timelimit)
    count = 0x7fffffff;
  else {
    count = timelimit * dsp_speed;
    if (dsp_stereo)
      count *= 2;
    if (samplesize != 8)
      count *= 2;
  }
  return count;
}

void main() {
  printf("Program Start\n");
  int i;
  int count = 0;
  while (1) {
    //recording part: 
	if (inw(na_switch_pio) == 0x3) {
  
		FILE * fd = fopen("/mnt/memstick/testfile.wav", "w"); 
	
		if (fd==NULL) {
			printf("Error opening file\n");
			exit(-1); 
		}
  		 
		//WAVE header calls
		unsigned long cnt;
		cnt = calc_count();
		printf("cnt =%d\n", cnt);
		start_wave(fd, cnt);
		
		//int numSamples = 32768; 
		short int * samples = (short *) malloc(sizeof(short) * numSamples); 
					
				for (i=0; i<numSamples; i++) { 
					//volatile unsigned int audioIn = inw(na_audio);
					samples[i] = IORD(na_audio, 1);
					
					//for (count = 0; count < DELAY2; count++){ 
					//}
				}

				int numSamplesWritten = fwrite(samples, sizeof(short), numSamples, fd); 
				printf("numSamplesWritten=%d\n", numSamplesWritten); 
				printf("sizeof(short)=%d bytes\n", sizeof(short));
				if (numSamplesWritten!=numSamples) { 
				printf("Error: only wrote %d of %d samples\n", numSamplesWritten, numSamples); 
				}
				printf("samples[128]=%d\n", samples[128]);
				printf("samples[129]=%d\n", samples[129]);
				
		fflush(fd); 
        free(samples); 
		fclose(fd);
		printf("Done recording!\n");   
		exit (EXIT_SUCCESS);
	}

	else if (inw(na_switch_pio) == 0x5) {
		FILE * fd = fopen("/mnt/memstick/testfile.wav", "r"); 
		if (fd==NULL) {
			printf("Error opening file\n");
			exit(-1); 
		}
		int i = 0;
		//int numSamples = 32768; 
		short int * sample = (short *) malloc(sizeof(short) * numSamples); 
		//typedef unsigned short sample[numSamples];
		int numSamplesRead;
		
			//keep reading until EOF
			//int audioOut; //set to na_audio (for output)
			printf("above fread\n");
			if ((numSamplesRead = fread(sample, sizeof(short), numSamples, fd))!= NULL) { 
				printf("inside fread\n");
				printf("sample[128]=%d\n", sample[128]);
				printf("sample[129]=%d\n", sample[129]);
				for (i=0; i<numSamples; i++) { 
					//for (count = 0; count < DELAY2; count++); //2nd delay
					//outw(sample[i], na_sram_0) ; 
					IOWR(na_audio, 1, sample[i]);
					//sample++;
				}
			}
			printf("numSamplesRead =%d\n", numSamplesRead);
			
		fflush(fd); 
        free(sample); 
		fclose(fd);
		printf("Done playing!\n");   
		exit (EXIT_SUCCESS);
	}
  }
}

